@@ -7,17 +7,25 @@ This module contains implementations for the Constraint{s} class.
 
 import re
 from dataclasses import dataclass
+from itertools import product
 from typing import Any, Sequence, Union
 
 import dask
 import numpy as np
 import pandas as pd
 import xarray as xr
+from numpy import array
 from scipy.sparse import coo_matrix
 from xarray import DataArray, Dataset
 
-from linopy.common import _merge_inplace, replace_by_map
-from linopy.expressions import LinearExpression
+from linopy import expressions, variables
+from linopy.common import (
+    _merge_inplace,
+    has_assigned_model,
+    has_optimized_model,
+    is_constant,
+    replace_by_map,
+)
 
 
 class Constraint(DataArray):
@@ -81,6 +89,7 @@ class Constraint(DataArray):
         return DataArray(self)
 
     @property
+    @has_assigned_model
     def coeffs(self):
         """
         Get the left-hand-side coefficients of the constraint.
@@ -88,27 +97,16 @@ class Constraint(DataArray):
         The function raises an error in case no model is set as a
         reference.
         """
-        if self.model is None:
-            raise AttributeError("No reference model is assigned to the constraint.")
         return self.model.constraints.coeffs[self.name]
 
     @coeffs.setter
+    @has_assigned_model
     def coeffs(self, value):
-        labels = self.model.constraints.labels
-        value = DataArray(value)
-        term_dim = self.name + "_term"
-
-        if term_dim not in value.dims:
-            value = value.expand_dims(term_dim)
-
-        assert (set(value.dims) - {term_dim}).issubset(labels.dims), (
-            "Dimensions of new values not a subset of labels dimensions, "
-            "therefore the new coefficients cannot be aligned with the existing labels."
-        )
-
+        value = DataArray(value).broadcast_like(self.vars)
         self.model.constraints.coeffs[self.name] = value
 
     @property
+    @has_assigned_model
     def vars(self):
         """
         Get the left-hand-side variables of the constraint.
@@ -116,27 +114,16 @@ class Constraint(DataArray):
         The function raises an error in case no model is set as a
         reference.
         """
-        if self.model is None:
-            raise AttributeError("No reference model is assigned to the constraint.")
         return self.model.constraints.vars[self.name]
 
     @vars.setter
+    @has_assigned_model
     def vars(self, value):
-        labels = self.model.constraints.labels
-        value = DataArray(value)
-        term_dim = self.name + "_term"
-
-        if term_dim not in value.dims:
-            value = value.expand_dims(term_dim)
-
-        assert (set(value.dims) - {term_dim}).issubset(labels.dims), (
-            "Dimensions of new values not a subset of labels dimensions, "
-            "therefore the new variables cannot be aligned with the existing labels."
-        )
-
+        value = DataArray(value).broadcast_like(self.coeffs)
         self.model.constraints.vars[self.name] = value
 
     @property
+    @has_assigned_model
     def lhs(self):
         """
         Get the left-hand-side linear expression of the constraint.
@@ -144,13 +131,15 @@ class Constraint(DataArray):
         The function raises an error in case no model is set as a
         reference.
         """
-        coeffs = self.coeffs.rename({self.name + "_term": "_term"})
-        vars = self.vars.rename({self.name + "_term": "_term"})
-        return LinearExpression(Dataset({"coeffs": coeffs, "vars": vars}))
+        term_dim = self.name + "_term"
+        coeffs = self.coeffs.rename({term_dim: "_term"})
+        vars = self.vars.rename({term_dim: "_term"})
+        return expressions.LinearExpression(Dataset({"coeffs": coeffs, "vars": vars}))
 
     @lhs.setter
+    @has_assigned_model
     def lhs(self, value):
-        if not isinstance(value, LinearExpression):
+        if not isinstance(value, expressions.LinearExpression):
             raise TypeError("Assigned lhs must be a LinearExpression.")
 
         value = value.rename(_term=self.name + "_term")
@@ -158,6 +147,7 @@ class Constraint(DataArray):
         self.vars = value.vars
 
     @property
+    @has_assigned_model
     def sign(self):
         """
         Get the signs of the constraint.
@@ -165,23 +155,19 @@ class Constraint(DataArray):
         The function raises an error in case no model is set as a
         reference.
         """
-        if self.model is None:
-            raise AttributeError("No reference model is assigned to the constraint.")
         return self.model.constraints.sign[self.name]
 
     @sign.setter
+    @has_assigned_model
+    @is_constant
     def sign(self, value):
-        labels = self.model.constraints.labels
-        value = DataArray(value)
-        assert set(value.dims).issubset(labels.dims), (
-            "Dimensions of new values not a subset of labels dimensions, "
-            "therefore the new signs cannot be aligned with the existing labels."
-        )
+        value = DataArray(value).broadcast_like(self)
         if (value == "==").any():
             raise ValueError('Sign "==" not supported, use "=" instead.')
         self.model.constraints.sign[self.name] = value
 
     @property
+    @has_assigned_model
     def rhs(self):
         """
         Get the right hand side constants of the constraint.
@@ -189,22 +175,19 @@ class Constraint(DataArray):
         The function raises an error in case no model is set as a
         reference.
         """
-        if self.model is None:
-            raise AttributeError("No reference model is assigned to the constraint.")
         return self.model.constraints.rhs[self.name]
 
     @rhs.setter
+    @has_assigned_model
+    @is_constant
     def rhs(self, value):
-        labels = self.model.constraints.labels
-        value = DataArray(value)
-        assert set(value.dims).issubset(labels.dims), (
-            "Dimensions of new values not a subset of labels dimensions, "
-            "therefore the new right-hand-side cannot be aligned with the existing labels."
-        )
-
+        if isinstance(value, (variables.Variable, expressions.LinearExpression)):
+            raise TypeError(f"Assigned rhs must be a constant, got {type(value)}).")
+        value = DataArray(value).broadcast_like(self)
         self.model.constraints.rhs[self.name] = value
 
     @property
+    @has_optimized_model
     def dual(self):
         """
         Get the dual values of the constraint.
@@ -212,10 +195,6 @@ class Constraint(DataArray):
         The function raises an error in case no model is set as a
         reference or the model status is not okay.
         """
-        if self.model is None:
-            raise AttributeError("No reference model is assigned to the constraint.")
-        if self.model.status != "ok":
-            raise AttributeError("Underlying model not optimized.")
         if not list(self.model.dual):
             raise AttributeError(
                 "Underlying is optimized but does not have dual values stored."
@@ -550,3 +529,134 @@ class Constraints:
             shape = (ncons, nvars)  # same as model.nvars/ncons but already there
 
         return coo_matrix((data, (rows, cols)), shape=shape)
+
+
+class AnonymousConstraint:
+    """
+    A constraint container used for storing multiple constraint arrays.
+    """
+
+    __slots__ = ("_lhs", "_sign", "_rhs")
+
+    def __init__(self, lhs, sign, rhs):
+        """
+        Initialize a anonymous constraint.
+        """
+        if isinstance(rhs, (variables.Variable, expressions.LinearExpression)):
+            raise TypeError(f"Assigned rhs must be a constant, got {type(rhs)}).")
+        self._lhs, self._rhs = xr.align(lhs, DataArray(rhs))
+        self._sign = DataArray(sign)
+
+    @property
+    def lhs(self):
+        return self._lhs
+
+    @property
+    def sign(self):
+        return self._sign
+
+    @property
+    def rhs(self):
+        return self._rhs
+
+    def __repr__(self):
+        """
+        Get the string representation of the expression.
+        """
+        lhs_string = self.lhs.to_dataset().__repr__()  # .split("\n", 1)[1]
+        lhs_string = lhs_string.split("Data variables:\n", 1)[1]
+        lhs_string = lhs_string.replace("    coeffs", "coeffs")
+        lhs_string = lhs_string.replace("    vars", "vars")
+        if self.sign.size == 1:
+            sign_string = self.sign.item()
+        else:
+            sign_string = self.sign.__repr__().split("\n", 1)[1]
+        if self.rhs.size == 1:
+            rhs_string = self.rhs.item()
+        else:
+            rhs_string = self.rhs.__repr__().split("\n", 1)[1]
+        return (
+            f"Anonymous Constraint:\n"
+            f"---------------------\n"
+            f"\n{lhs_string}"
+            f"\n{sign_string}"
+            f"\n{rhs_string}"
+        )
+
+    def from_rule(model, rule, coords):
+        """
+        Create a constraint from a rule and a set of coordinates.
+
+        This functionality mirrors the assignment of constraints as done by
+        Pyomo.
+
+
+        Parameters
+        ----------
+        model : linopy.Model
+            Passed to function `rule` as a first argument.
+        rule : callable
+            Function to be called for each combinations in `coords`.
+            The first argument of the function is the underlying `linopy.Model`.
+            The following arguments are given by the coordinates for accessing
+            the variables. The function has to return a
+            `AnonymousScalarConstraint`. Therefore use the direct getter when
+            indexing variables in the linear expression.
+        coords : coordinate-like
+            Coordinates to processed by `xarray.DataArray`.
+            For each combination of coordinates, the function given by `rule` is called.
+            The order and size of coords has to be same as the argument list
+            followed by `model` in function `rule`.
+
+
+        Returns
+        -------
+        linopy.AnonymousConstraint
+
+        Examples
+        --------
+        >>> from linopy import Model, LinearExpression
+        >>> m = Model()
+        >>> coords = pd.RangeIndex(10), ["a", "b"]
+        >>> x = m.add_variables(0, 100, coords)
+        >>> def bound(m, i, j):
+        ...     if i % 2:
+        ...         return (i - 1) * x[i - 1, j] >= 0
+        ...     else:
+        ...         return i * x[i, j] >= 0
+        ...
+        >>> con = AnonymousConstraint.from_rule(m, bound, coords)
+        >>> con = m.add_constraints(con)
+        """
+        if not isinstance(coords, xr.core.dataarray.DataArrayCoordinates):
+            coords = DataArray(coords=coords).coords
+        shape = list(map(len, coords.values()))
+
+        # test output type
+        output = rule(model, *[c.values[0] for c in coords.values()])
+        if not isinstance(output, AnonymousScalarConstraint):
+            msg = f"`rule` has to return AnonymousScalarConstraint not {type(output)}."
+            raise TypeError(msg)
+
+        combinations = product(*[c.values for c in coords.values()])
+        cons = [rule(model, *coord) for coord in combinations]
+        exprs = [con.lhs for con in cons]
+
+        lhs = expressions.LinearExpression._from_scalarexpression_list(exprs, coords)
+        sign = DataArray(array([c.sign for c in cons]).reshape(shape), coords)
+        rhs = DataArray(array([c.rhs for c in cons]).reshape(shape), coords)
+        return AnonymousConstraint(lhs, sign, rhs)
+
+
+@dataclass
+class AnonymousScalarConstraint:
+    """
+    Container for anonymous scalar constraint.
+
+    This contains a left-hand-side (lhs), a sign and a right-hand-side
+    (rhs) for exactly one constraint.
+    """
+
+    lhs: expressions.ScalarLinearExpression
+    sign: str
+    rhs: float
