@@ -204,7 +204,7 @@ def to_block_files(m, fn):
     m.calculate_block_maps()
 
     N = int(m.blocks.max())
-    for n in range(N + 1):
+    for n in range(N + 2):
         (path / f"block{n}").mkdir()
 
     vars = m.variables
@@ -215,7 +215,7 @@ def to_block_files(m, fn):
     for key, suffix in zip(["labels", "lower", "upper"], ["x", "xl", "xu"]):
         arr = vars.ravel(key, filter_missings=True)
         for n in tqdm(range(N + 1), desc=f"Write variable {key}"):
-            arr[blocks == n].tofile(path / f"block{n}" / suffix)
+            arr[blocks == n].tofile(path / f"block{n}" / suffix, sep="\n")
 
     # Write out objective (uses variable blocks from above)
     coeffs = np.zeros(m._xCounter)
@@ -223,54 +223,66 @@ def to_block_files(m, fn):
     # reorder like non-missing variables
     coeffs = coeffs[vars.ravel("labels", filter_missings=True)]
     for n in tqdm(range(N + 1), desc="Write objective"):
-        coeffs[blocks == n].tofile(path / f"block{n}" / "c")
+        coeffs[blocks == n].tofile(path / f"block{n}" / "c", sep="\n")
 
     # Write out rhs
     blocks = cons.ravel("blocks", filter_missings=True)
     rhs = cons.ravel("rhs", filter_missings=True)
-    is_equality = cons.ravel(cons.sign == "==", filter_missings=True)
+    is_equality = cons.ravel(cons.sign == "=", filter_missings=True)
     is_lower_bound = cons.ravel(cons.sign == ">=", filter_missings=True)
 
-    for n in tqdm(range(N + 1), desc="Write RHS"):
+    for n in tqdm(range(N + 2), desc="Write RHS"):
         is_blockn = blocks == n
 
-        rhs[is_blockn & is_equality].tofile(path / f"block{n}" / "b")
+        rhs[is_blockn & is_equality].tofile(path / f"block{n}" / "b", sep="\n")
 
         not_equality = is_blockn & ~is_equality
         is_lower_bound_sub = is_lower_bound[not_equality]
         rhs_sub = rhs[not_equality]
 
         lower_bound = np.where(is_lower_bound_sub, rhs_sub, -np.inf)
-        lower_bound.tofile(path / f"block{n}" / "dl")
+        lower_bound.tofile(path / f"block{n}" / "dl", sep="\n")
 
         upper_bound = np.where(~is_lower_bound_sub, rhs_sub, np.inf)
-        upper_bound.tofile(path / f"block{n}" / "du")
+        upper_bound.tofile(path / f"block{n}" / "du", sep="\n")
 
     # Write out constraints
     conblocks = cons.ravel("blocks", "vars", filter_missings=True)
     varblocks = cons.ravel("var_blocks", "vars", filter_missings=True)
-    is_equality = cons.ravel(cons.sign == "==", "vars", filter_missings=True)
+    is_equality = cons.ravel(cons.sign == "=", "vars", filter_missings=True)
 
     is_varblock_0 = varblocks == 0
-    is_conblock_L = conblocks == N
+    is_conblock_L = conblocks == N + 1
 
     for key, suffix in zip(["labels", "coeffs", "vars"], ["row", "data", "col"]):
         arr = cons.ravel(key, "vars", filter_missings=True)
         for n in tqdm(range(N + 1), desc=f"Write constraint {key}"):
+
             is_conblock_n = conblocks == n
             is_varblock_n = varblocks == n
 
-            mask = is_conblock_n & is_varblock_n
-            arr[mask & is_equality].tofile(path / f"block{n}" / f"B_{suffix}")
-            arr[mask & ~is_equality].tofile(path / f"block{n}" / f"D_{suffix}")
-
             mask = is_conblock_n & is_varblock_0
-            arr[mask & is_equality].tofile(path / f"block{n}" / f"A_{suffix}")
-            arr[mask & ~is_equality].tofile(path / f"block{n}" / f"C_{suffix}")
+            arr[mask & is_equality].tofile(path / f"block{n}" / f"A_{suffix}", sep="\n")
+            arr[mask & ~is_equality].tofile(
+                path / f"block{n}" / f"C_{suffix}", sep="\n"
+            )
 
             mask = is_conblock_L & is_varblock_n
-            arr[mask & is_equality].tofile(path / f"block{n}" / f"BL_{suffix}")
-            arr[mask & ~is_equality].tofile(path / f"block{n}" / f"DL_{suffix}")
+            arr[mask & is_equality].tofile(
+                path / f"block{n}" / f"BL_{suffix}", sep="\n"
+            )
+            arr[mask & ~is_equality].tofile(
+                path / f"block{n}" / f"DL_{suffix}", sep="\n"
+            )
+
+            if n:
+                mask = is_conblock_n & is_varblock_n
+                arr[mask & is_equality].tofile(
+                    path / f"block{n}" / f"B_{suffix}", sep="\n"
+                )
+                arr[mask & ~is_equality].tofile(
+                    path / f"block{n}" / f"D_{suffix}", sep="\n"
+                )
 
 
 def non_bool_dict(d):
