@@ -20,15 +20,21 @@ from numpy import floating, inf, issubdtype
 from xarray import DataArray, Dataset, zeros_like
 
 import linopy.expressions as expressions
-from linopy.common import _merge_inplace
+from linopy.common import (
+    _merge_inplace,
+    has_assigned_model,
+    has_optimized_model,
+    is_constant,
+)
 
 
 def varwrap(method, *default_args, **new_default_kwargs):
     @functools.wraps(method)
-    def _varwrap(*args, **kwargs):
+    def _varwrap(obj, *args, **kwargs):
         for k, v in new_default_kwargs.items():
             kwargs.setdefault(k, v)
-        return Variable(method(*default_args, *args, **kwargs))
+        obj = DataArray(obj)
+        return Variable(method(obj, *default_args, *args, **kwargs))
 
     _varwrap.__doc__ = f"Wrapper for the xarray {method} function for linopy.Variable"
     if new_default_kwargs:
@@ -48,44 +54,39 @@ class Variable(DataArray):
 
     Examples
     --------
+    >>> from linopy import Model
+    >>> import pandas as pd
     >>> m = Model()
     >>> x = m.add_variables(pd.Series([0, 0]), 1, name="x")
     >>> y = m.add_variables(4, pd.Series([8, 10]), name="y")
 
     Add variable together:
 
-    >>> x + y
-
-    ::
-
-        Linear Expression with 2 term(s):
-        ----------------------------------
-
-        Dimensions:  (dim_0: 2, _term: 2)
-        Coordinates:
-            * dim_0    (dim_0) int64 0 1
-            * _term    (_term) int64 0 1
-        Data:
-            coeffs   (dim_0, _term) int64 1 1 1 1
-            vars     (dim_0, _term) int64 1 3 2 4
-
+    >>> x + y  # doctest: +SKIP
+    Linear Expression with 2 term(s):
+    ----------------------------------
+    <BLANKLINE>
+    Dimensions:  (dim_0: 2, _term: 2)
+    Coordinates:
+      * dim_0    (dim_0) int64 0 1
+    Dimensions without coordinates: _term
+    Data:
+        coeffs   (dim_0, _term) int64 1 1 1 1
+        vars     (dim_0, _term) int64 0 2 1 3
 
     Multiply them with a coefficient:
 
-    >>> 3 * x
-
-    ::
-
-        Linear Expression with 1 term(s):
-        ----------------------------------
-
-        Dimensions:  (dim_0: 2, _term: 1)
-        Coordinates:
-            * _term    (_term) int64 0
-            * dim_0    (dim_0) int64 0 1
-        Data:
-            coeffs   (dim_0, _term) int64 3 3
-            vars     (dim_0, _term) int64 1 2
+    >>> 3 * x  # doctest: +SKIP
+    Linear Expression with 1 term(s):
+    ----------------------------------
+    <BLANKLINE>
+    Dimensions:  (dim_0: 2, _term: 1)
+    Coordinates:
+      * dim_0    (dim_0) int64 0 1
+    Dimensions without coordinates: _term
+    Data:
+        coeffs   (dim_0, _term) int64 3 3
+        vars     (dim_0, _term) int64 0 1
 
 
     Further operations like taking the negative and subtracting are supported.
@@ -114,6 +115,20 @@ class Variable(DataArray):
     __array_ufunc__ = None
     __array_priority__ = 10000
 
+    def __getitem__(self, keys) -> "ScalarVariable":
+        keys = (keys,) if not isinstance(keys, tuple) else keys
+        assert all(map(np.isscalar, keys)), (
+            "The get function of Variable is different as of xarray.DataArray. "
+            "Set single values for each dimension in order to obtain a "
+            "ScalarVariable. For all other purposes, use `.sel` and `.isel`."
+        )
+        if not self.ndim:
+            return ScalarVariable(self.data.item())
+        assert self.ndim == len(keys), f"expected {self.ndim} keys, got {len(keys)}."
+        key = dict(zip(self.dims, keys))
+        selector = [self.get_index(k).get_loc(v) for k, v in key.items()]
+        return ScalarVariable(self.data[tuple(selector)])
+
     def to_array(self):
         """
         Convert the variable array to a xarray.DataArray.
@@ -235,6 +250,7 @@ class Variable(DataArray):
         return self.to_linexpr().rolling_sum(**kwargs)
 
     @property
+    @has_assigned_model
     def upper(self):
         """
         Get the upper bounds of the variables.
@@ -242,11 +258,11 @@ class Variable(DataArray):
         The function raises an error in case no model is set as a
         reference.
         """
-        if self.model is None:
-            raise AttributeError("No reference model is assigned to the variable.")
         return self.model.variables.upper[self.name]
 
     @upper.setter
+    @has_assigned_model
+    @is_constant
     def upper(self, value):
         """
         Set the upper bounds of the variables.
@@ -254,18 +270,11 @@ class Variable(DataArray):
         The function raises an error in case no model is set as a
         reference.
         """
-
-        if self.model is None:
-            raise AttributeError("No reference model is assigned to the variable.")
-        labels = self.model.variables.labels
-        value = DataArray(value)
-        assert set(value.dims).issubset(
-            labels.dims
-        ), "Dimensions of new values not a subset of labels dimensions."
-
+        value = DataArray(value).broadcast_like(self)
         self.model.variables.upper[self.name] = value
 
     @property
+    @has_assigned_model
     def lower(self):
         """
         Get the lower bounds of the variables.
@@ -273,11 +282,11 @@ class Variable(DataArray):
         The function raises an error in case no model is set as a
         reference.
         """
-        if self.model is None:
-            raise AttributeError("No reference model is assigned to the variable.")
         return self.model.variables.lower[self.name]
 
     @lower.setter
+    @has_assigned_model
+    @is_constant
     def lower(self, value):
         """
         Set the lower bounds of the variables.
@@ -285,17 +294,11 @@ class Variable(DataArray):
         The function raises an error in case no model is set as a
         reference.
         """
-        if self.model is None:
-            raise AttributeError("No reference model is assigned to the variable.")
-        labels = self.model.variables.labels
-        value = DataArray(value)
-        assert set(value.dims).issubset(
-            labels.dims
-        ), "Dimensions of new values not a subset of labels dimensions."
-
+        value = DataArray(value).broadcast_like(self)
         self.model.variables.lower[self.name] = value
 
     @property
+    @has_optimized_model
     def sol(self):
         """
         Get the optimal values of the variable.
@@ -303,8 +306,6 @@ class Variable(DataArray):
         The function raises an error in case no model is set as a
         reference or the model is not optimized.
         """
-        if self.model is None:
-            raise AttributeError("No reference model is assigned to the variable.")
         if self.model.status != "ok":
             raise AttributeError("Underlying model not optimized.")
         return self.model.solution[self.name]
@@ -382,6 +383,8 @@ class Variable(DataArray):
 
     roll = varwrap(DataArray.roll)
 
+    rolling = varwrap(DataArray.rolling)
+
 
 @dataclass(repr=False)
 class Variables:
@@ -606,3 +609,41 @@ class Variables:
             res[np.ravel(labels)] = np.ravel(block_map[name])
         res[-1] = -1
         return res
+
+
+@dataclass
+class ScalarVariable:
+    label: int
+    coords: dict = None
+
+    def to_linexpr(self, coeff=1):
+        if not isinstance(coeff, (int, float)):
+            raise TypeError(f"Coefficient must be a numeric value, got {type(coeff)}.")
+        return expressions.ScalarLinearExpression((coeff,), (self.label,))
+
+    def __neg__(self):
+        return self.to_linexpr(-1)
+
+    def __add__(self, other):
+        return self.to_linexpr(1) + other
+
+    def __sub__(self, other):
+        return self.to_linexpr(1) - other
+
+    def __mul__(self, coeff):
+        return self.to_linexpr(coeff)
+
+    def __rmul__(self, coeff):
+        return self.to_linexpr(coeff)
+
+    def __div__(self, coeff):
+        return self.to_linexpr(1 / coeff)
+
+    def __le__(self, other):
+        return self.to_linexpr(1).__le__(other)
+
+    def __ge__(self, other):
+        return self.to_linexpr(1).__ge__(other)
+
+    def __eq__(self, other):
+        return self.to_linexpr(1).__eq__(other)
