@@ -12,7 +12,23 @@ import pytest
 import xarray as xr
 
 import linopy
-from linopy import LinearExpression, Model
+from linopy import Model
+
+
+def test_variable_getter():
+    m = Model()
+    x = m.add_variables(coords=[range(10)])
+
+    assert isinstance(x[0], linopy.variables.ScalarVariable)
+
+    with pytest.raises(AssertionError):
+        x[0, 0]
+
+    with pytest.raises(AssertionError):
+        x[0:5]
+
+    with pytest.raises(AssertionError):
+        x[[1, 2, 3]]
 
 
 def test_variable_repr():
@@ -121,7 +137,6 @@ def test_variable_type_preservation():
 
 
 def test_variable_getter_without_model():
-
     data = xr.DataArray(range(10)).rename("var")
     v = linopy.variables.Variable(data)
 
